from chatqhelper.mqtt import MqttClient
from chatqhelper import scheduler
from chatqhelper import debug
import time


logger = debug.logger(__name__)


class MqttHandler:
    def __init__(self, interval=1):
        self._mqtt_client = MqttClient.create(self._on_mqtt_connect, handle_in_thread=True)
        self._interval = interval

    @property
    def mqtt(self):
        return self._mqtt_client

    def start(self):
        self._mqtt_client.loop_start()
        self.loop()

    def loop(self):
        while True:
            time.sleep(self._interval)
            self._loop_imp()

    def _loop_imp(self):
        pass

    def stop(self):
        self._mqtt_client.loop_stop()

    def restart(self):
        self.stop()
        self.start()

    def _on_mqtt_connect(self, client, userdata, flags, rc):
        logger.info('connected to solace with code ' + str(rc))


class MqttAndScheduleHandler(MqttHandler):
    def __init__(self):
        super().__init__()
        self._scheduler = scheduler.UTCScheduler()

    @property
    def scheduler(self):
        return self._scheduler

    def start(self):
        self._scheduler.loop_start()
        super().start()

    def stop(self):
        self._scheduler.loop_stop()
        super().stop()



import os

HOSTNAME = os.environ.get('HOSTNAME')
SERVICE_NAME = '-'.join(HOSTNAME.split('-')[:2])
SLACK_ERR_CHANNEL = 'backend-err-' + os.environ.get('APP_CONTEXT','local')
SLACK_PERFORMANCE_CHANNEL = 'backend-perf-' + os.environ.get('APP_CONTEXT','local')


#TOPICS
TOPIC_PERFORMANCE_SUCCESS = "performance/success/backend"
TOPIC_PERFORMANCE_SUBSCRIBE = "performance/sub/backend"
TOPIC_PERFORMANCE = "performance/+/backend"
TOPIC_BACKEND_EXCEPTION = "exceptions/backend"

STATUS_SUCCESS = 'success'
STATUS_FAILURE = 'failure'



from functools import wraps
from chatqhelper.common import constants
import time
import json


class PerformanceTracking:
    """Caculate time to completion for a task will publish to performance-report/backend topic
    """
    @staticmethod
    def subscribe(client, topic):
        client.publish(
            constants.TOPIC_PERFORMANCE_SUBSCRIBE,
            json.dumps({'topic':topic})
        )
    
    @staticmethod
    def track(func):
        @wraps(func)
        def _wraper(client, message):  
            start_time = time.process_time()
            func(client, message)
            end_time = time.process_time()
            report = {'topic': message.topic, 'start_time': start_time, 'end_time': end_time}
            client.publish(
                constants.TOPIC_PERFORMANCE_SUCCESS,
                json.dumps(report)
            )
        return _wraper